% Function to design optimal FIR filter using fir2 in signal processing
%   toolbox (implemented via Cook and Miller Optimal Digital Filter with 
%   hamming option applied when truncating filter to final filter order)
%
% Requires Signal Processing Toolbox to be installed.
%
% Usage:
%   Maximal Specification:
%      [f_weights, f_axis, gain, ideal_actual_corr] = FIR_design(lowcut, highcut, srate, notch, plot_gain, filtorder_factor);
%   Minimal Specification:
%       f_weights = FIR_design(lowcut, highcut, srate, plot_gain, filtorder_factor);
%   Some of the many Optional Specifications:
%       f_weights = FIR_design(lowcut, highcut, srate, 1); % make notch filter
%       f_weights = FIR_design(lowcut, highcut, srate, 0, 1); % make LP/HP/BP filter with plot of frequency response
%       f_weights = FIR_design(lowcut, highcut, srate, 1, 0, 3); % make notch filter with filter order 3 times the sample rate
%
% Where:
%   lowcut is lower-end cutoff (hz); for low pass filter use 0
%   high cut is upper-end cutoff (hz); for high pass filter use 0
%   srate is sample rate (hz)
%   notch is logical notch out frequencies, 0 = no (default for bandpass, lowpass, highpass), 1 = yes (notch) 
%   plot_gain is optional freq response plot, 0 = no (default), 1 = yes
%   filtorder_factor is a multipler to change filter length relative to sample rate (default = 1, where filter order = sample rate)
%
% Outputs:
%   f_weights: filter weights to apply with convolution method
%   f_axis: frequency axis for frequency response
%   gain: gain at frequencies of f_axis
%   ideal_actual_corr: correlation between ideal transfer function and
%                      actual transfer function -- if below .95 you best
%                      plot the functions and see if the filter is adequate
%
% Notes:
%   Default final filter order is one second of data (plus one point)
%   Becuase of this, this filter is best applied to continuous data
%   With Epoch data, and padding, filter will work but will produce edge 
%        effects that should be examined by plotting data
%
% Version 0.9
%
% Written July, 2011, updated Mar 2014, by John JB Allen jallen@u.arizona.edu
%


function [f_weights, f_axis, gain, ideal_actual_corr] = FIR_design(lowcut, highcut, srate, varargin)

% set defaults
plotit = 0;
filtorder_factor = 1;

% check to see all input arguments provided, and if not give documentation
if nargin == 6
    filtorder_factor = varargin{3};
    plotit = varargin{2};
    notch = varargin{1};
elseif nargin == 5
    plotit = varargin{2};
    notch = varargin{1};
elseif nargin == 4
    notch = varargin{1};
elseif nargin == 3
    notch = 0;
else
    doc FIR_design;
    disp('Minimum of three input parameters required: lowcut, highcut, srate');
    return
end;

% check for problems with nyquist and input freqs
if highcut == 0
    highcut = srate/2;
elseif highcut > srate/2
    error(sprintf('The highcut of %5.2f exceeds the Nyquist of %5.2f hz\n',highcut,srate/2));
elseif lowcut > highcut
    error(sprintf('The lowcut of %5.2f exceeds the highcut of %5.2f hz\n',lowcut, highcut));
end

% check for signal processing toolbox
if exist('fir2','file') ~= 2
      error('Optimal FIR Toolbox requies the Signal Processing Toolbox, which cannot be found in your environment!');
end


% set up parameters -- make FFT array dependent on srate
NumInFFTArray = 2*(2^nextpow2(srate)); % twice the next power of 2 that is >= srate
if mod(srate,2) == 0 % check to see if sample rate is even, since we want a filter with odd number points
    filt_order = round(srate*filtorder_factor); % filter length will be one second of data by default, or scaled by filtorder_factor (and fir2 adds a point)
else
    filt_order = round((srate-1)*filtorder_factor);
end

% define frequency vector from 0 to 1, where 1 is nyquist
T = NumInFFTArray/srate;  % length of epoch in time
fs = 0:1/T:srate/2;     % frequency vector from 0 to Nyquist
f = fs./(srate/2); % normalized frequency vector from 0 to Nyquist 

% Define ideal frequency function
m = fs >= lowcut & fs <= highcut;   % where the frequency is between cutpoints
if notch == 1
    m = ~m;    % invert pass and stop freqs for notch filter
end
m=double(m);

% Eureka -- now generate filter!
f_weights = fir2(filt_order,f,m); % see doc fir2 -- specify input frequencies (f: normalized 0 to nyquist) and ideal freq rsps (m: same length as f)
                                  %                 and fir2 obtains filter coefficients are by applying an inverse fast Fourier transform to the
                                  %                 ideal freq response, and then windowing the result using a Hamming window.
                                  %                 This is the procedure outlined by Cook & Miller, 1991

% Freq Response
[h,w] = freqz(f_weights); % h is frequency response, w is frequencies corresponding to h
gain = abs(h);  % only need magnitude of h
f_axis = w/pi*(srate/2); % convert w back to frequencies (since w is in in terms of spacing on unit circle)

% Correlation between ideal and actual
% Because filt order is not equal to initial array size, need to resample
% to make the correlation work...
ideal_actual_corr = corr(m',resample(abs(h),length(fs),length(h)));

% optional plot
if plotit
    figure('numbertitle','off','Name','Transfer function of optimal FIR versus ideal');
    plot(fs,m,'-k');
    hold on;
    plot(w/pi*(srate/2),abs(h),'color',[0.5 0 1]);
    legend('Ideal',['Opt FIR r_i_d_e_a_l=' sprintf('%4.3f',ideal_actual_corr)]);
    title(['Comparison of ' sprintf('%5.2f-%5.2f hz',lowcut,highcut) ' Frequency Response Magnitudes at ' num2str(srate) ' hz sample rate']);
    xlabel 'Hz';
    ylabel 'Gain';
    ylim([0 1.4]);
end

